/*  Testing connected components in the grid.  Provides for	*/
/*  region coloring and plotting of result using grcplot.	*/
/*  File:  grcdmp.cs						*/


#include <stdio.h>
#include <stdlib.h>
#include <cscomm.h>
#include "grccsw.hs"
#include "grclr.hs"
#include "Xu.hcs"
#include "Xuf.hcs"
#include "grcplt.hcs"


/* Draw horizontal line in grid according to b in row i,	*/
/* starting in column j for c columns.				*/

void bline(bool:current *b, int i, int j, int c)
 { int k;
   putchar('+');
   for ( k = 0; k < c; k++, j++ )
    {     if ( [i][j]*b  ) printf("-----+");
     else		   printf("     +");
    }
   printf("\n");
 }

/* Print a portion of the current grid showing barriers. 	*/
/* Uses cx for component numbers, nb, sb, wb, eb for barriers.	*/

void dcc(int:cells *cx, int i, int j, int r, int c)
 { int m, n, k;
   m = i + r;  n = j + c;	/* Row, column limits */
   bline(&nb, i, j, c);
   for ( ; i < m; i++ )		/* Draw row i */
    { putchar( [i][j]wb ? '|' : ' ');
      for ( k = j; k < n; k++ )
       { printf("%5d", [i][k]*cx);
	 putchar( [i][k]eb ? '|' : ' ');
       }
      printf("\n");
      bline(&sb, i, j, c);
    }
 }

/* Check that neighbors on axis ax, distance ds are legal */

int:cells clgl(int:cells cl, int ax, int ds)
 { int:cells ccx, clx, e1, e2;
   ccx = from_grid_dim(&cc, cc, ax, ds);
   clx = from_grid_dim(&cl, cl, ax, ds);
   printf("cc's:  %5d %5d  cl's:  %5d %5d\n",
	  [0][7]cc, [0][7]ccx, [0][7]cl, [0][7]clx);
   e1 = (cc == ccx);  e2 = (cl == clx);
   return ( (e1 == e2) | (cl == 0) | (clx == 0) );
 }

/* Check coloring recorded in clr for labelling cc.	*/
/* Call with cells active				*/

int:cells okc;

void clrchk(int:cells clr)
 { int nc, nu, ok, bi, bj, bn;
   nc = (>?= clr);		/* No. of colors used     */
   where ( clr == 0 )		/* No. of uncolored cells */
     nu = (+= (int:cells) 1);
   okc = (  clgl(clr, 0, 1) & clgl(clr, 0, -1)
	  & clgl(clr, 1, 1) & clgl(clr, 1, -1));
   ok = (&= okc);
   printf("%d colors used, %d cells uncolored, ok: %d\n",
	   nc,             nu,                     ok    );
   if ( !ok )
    { where ( !okc )
       { bi = (<?= x);
	 bn = (+= (int:current) 1);
         where ( bi == x ) bj = (<?= y);
       }
      printf("Failure at [%d][%d],  %d failures in all.\n",
		          bi, bj,   bn			   );
    }
 }

/* Main program -- manages tests and timing interactively.	*/
/* Responses:							*/
/*  p <dd>		- run test with probability dd [0..100]	*/
/*  o <dd>              - animate spreading with probability dd */
/*  w <i> <j> <r> <c>	- print [i..]i+r by [j..]j+c  (r by c)	*/
/*  c <n>		- color using n colors, n <= 32		*/
/*  k <n>               - Animated Map Coloring using n colors  */
/*  v <i> <j> <r> <c>	- print colors as for w			*/
/*  e <i> <j> <r> <c>	- print okc as for w			*/
/*  h			- help
/*  q			- quit					*/

void main()
 { char c[2];		/* For input character     */
   int:cells clr;	/* Coloring		   */
   int  pr,		/* Probability in [0..100] */
        bs,		/* Number of barriers	   */
        nc,		/* Number of cycles	   */




        nr,		/* Number of regions	   */
	Nx,		/* Max no. of colors	   */
	i, j, rw, cl, cy;	/* Rectangle to print	   */
   printf("Grid is %d by %d.\n", XMX, YMX);
   with (cells) setup(0);	/* Initialize representation */
   MkScreen();  getcolors(&Nx);
   /* Read - execute - print loop */
   while ( printf("grid cc> "), scanf("%1s", c), c[0] != 'q' )
    switch ( c[0] )
     { case 'p':  scanf("%d", &pr);
       with (cells) everywhere
	 { bs = setup(pr);  Barriers();  Plotbgrd();
	   CM_timer_clear(1);  CM_timer_start(1);
	   nc = gccs();
	   CM_timer_stop(1);  CM_timer_print(1);
	   nr = (+= (cc == (y + YMX*x)));
	 }
       printf("%d barriers, %d cycles, %d regions.\n", bs, nc, nr);
       break;
     case 'o':  scanf("%d", &pr);
       with (cells) everywhere
	 { bs = setup(pr);  Barriers();  Plotbgrd();
	   CM_timer_clear(1);  CM_timer_start(1);
	   printf ("\n***** Just figured out Final colors! *****\n");
	   nc = gccs();		/* Added by ES - 13 May 1995 */
	   clr = grcolor(cc, cl, &cy);
	   CM_timer_stop(1);  CM_timer_print(1);
	   CM_timer_clear(1);  CM_timer_start(1);
	   nc = gccs_growing(cc, cl, &cy, clr);
	   CM_timer_stop(1);  CM_timer_print(1);
	   nr = (+= (cc == (y + YMX*x)));
	 }
       printf("%d barriers, %d cycles, %d regions.\n", bs, nc, nr);
       break;
     case 'k':  scanf("%d", &cl);	/* No. of colors */
       with (cells) everywhere
	 { CM_timer_clear(1);  CM_timer_start(1);
	   clr = grcolor_moving(cc, cl, &cy);
	   CM_timer_stop(1);  CM_timer_print(1);
	   printf("%d cycles used for coloring.\n", cy);
	   Plotgclr(clr);
	   clrchk(clr);
	 }
       break;
     case 'c':  scanf("%d", &cl);	/* No. of colors */
       with (cells) everywhere
	 { CM_timer_clear(1);  CM_timer_start(1);
	   clr = grcolor(cc, cl, &cy);
	   CM_timer_stop(1);  CM_timer_print(1);
	   printf("%d cycles used for coloring.\n", cy);
	   Plotgclr(clr);
	   clrchk(clr);
	 }
       break;
     case 'w':  scanf("%d%d%d%d", &i, &j, &rw, &cl);
       dcc(&cc, i, j, rw, cl);
       break;
     case 'v':  scanf("%d%d%d%d", &i, &j, &rw, &cl);
       dcc(&clr, i, j, rw, cl);
       break;
     case 'e':  scanf("%d%d%d%d", &i, &j, &rw, &cl);
       dcc(&okc, i, j, rw, cl);
       break;
     default:  printf("Bad input -- ignored.\n");  break;
     }
 }
     
